<?php 

namespace Teknisa\DF\Modules\Entrada\Services;

/* Bibliotecas */
use Doctrine\ORM\EntityManager;
use Geral\Entities\Empresa;
use Geral\Entities\Inscrestad;
use NFePHP\Common\Certificate;
use NFePHP\Common\Configure\Configure;
use NFePHP\Common\Exception\RuntimeException;
use NFePHP\Common\Files\FilesFolders;
use NFePHP\NFe\Common\Config;
use NFePHP\NFe\Tools as NFe;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Model\Fiscal\Entities\Arqxmlnfe;
use Teknisa\Model\Fiscal\Entities\Notadestinada;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Fiscal\Entities\Lotenfe;
use Teknisa\Model\Geral\Entities\Paramempresa;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

/* Classes DF */
use Teknisa\DF\Modules\Entrada\Factories\NumeracaoTransmissaoNfeFactory;
use Teknisa\DF\Services\NFe\ReturnNFe;

class NumeracaoTransmissaoNfeService {
    
    protected $numeracaoTransmissaoNfeFactory;
    protected $novoCodigo;
    protected $entityManager;
    protected $environment;
    protected $erros = [];
    
    public function __construct(NumeracaoTransmissaoNfeFactory $numeracaoTransmissaoNfeFactory, $novoCodigo, $entityManager, $environment, $fileServerLink) {
        $this->numeracaoTransmissaoNfeFactory = $numeracaoTransmissaoNfeFactory;
        $this->novoCodigo           = $novoCodigo;
        $this->entityManager        = $entityManager;
        $this->environment          = $environment;
        $this->masks = new Masks();
        $this->fileServerLink       = $fileServerLink;
        set_time_limit(0);
    }

    public function getSerieNfe($row) { 
        $CDEMPRESA = $row["CDEMPRESA"];
        $CDINSCESTA = $row["CDINSCESTA"];
        $NRINSJURIEST = $row["NRINSJURIEST"];
        $defaultFilter = $this->numeracaoTransmissaoNfeFactory->getSerieNfe($CDEMPRESA,$CDINSCESTA,$NRINSJURIEST);
        return $defaultFilter;
    }
    
    public function getNovoCodigo($row) {
        $indice = 0;
        $datasets = [];
        foreach($row as $value){
            $CDEMPRESA       =  $value["CDEMPRESA"];
            $NRINSJURIEST    =  $value["NRINSJURIEST"];
            $CDSERIENFE      =  $value["CDSERIENFE"];
            $IDTPAMBIENTENFE =  $value["IDTPAMBIENTENFE"];
            $defaultFilter = $this->numeracaoTransmissaoNfeFactory->getNovoCodigo($CDEMPRESA,$NRINSJURIEST,$CDSERIENFE,$IDTPAMBIENTENFE);
            if(count($defaultFilter) > 0) {
                $datasets[] = $defaultFilter[0];
            }
            else {
                $datasets[$indice]["CDCONTADOR"]   = ""; 
                $datasets[$indice]["NRSEQUENCIAL"] = "00000000000000000000";
                $datasets[$indice]["SERIENFE"]     = floatval($value["CDSERIENFE"]);
                $datasets[$indice]["TPAMBIENTE"]   = floatval($value["IDTPAMBIENTENFE"]);
            }
            $indice++;
        }
        return $datasets;
    }
    
    public function saveNumeracaoInicialTransmissaoNfe($row){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $intNovoCodigo = (intval($row['NRSEQUENCIAL'])); 
        $intAntigoCodigo = (intval($row['NRSEQUENCIALORI']));
        
        try{
            if ($intNovoCodigo<=$intAntigoCodigo)
                throw new \Exception('Nº da Nota inválido.');
            if ($row['CDCONTADOR'] == ""){
                $cdcontador = 'NOTAFISCALELE' . $row['CDEMPRESA'] . $row['NRINSJURIEST'] . $row['CDSERIENFE'] . $row['TPAMBIENTE'];
                $this->numeracaoTransmissaoNfeFactory->insereNovoCodigo($cdcontador, str_pad(($intNovoCodigo), 20, 0, STR_PAD_LEFT));
            }else{
                $cdcontador = $row['CDCONTADOR'];
                $this->numeracaoTransmissaoNfeFactory->alteraNovoCodigo($cdcontador, str_pad(($intNovoCodigo), 20, 0, STR_PAD_LEFT));
            }
            $NRPROTOCINUTNFE   = "TEKNISA";
            $IDTPEMISINUTIL    = "1";
            $DSJUSNUMINUTILNFE = "Rotina de Numeração Inicial de Transmissão (DFE27500)";
            
            for($ind=1; $ind<=($intNovoCodigo-$intAntigoCodigo); $ind++){
                $numInutilnfe = $this->numeracaoTransmissaoNfeFactory->verificaNumInutilNFEle($row['CDEMPRESA'], $row['NRINSJURIEST'], $row['CDSERIENFE'], str_pad(($intAntigoCodigo+$ind), 9, 0, STR_PAD_LEFT), $row['TPAMBIENTE']);
                if(count($numInutilnfe) == 0)
                    $this->numeracaoTransmissaoNfeFactory->insereNumInutilNFEle($row['CDEMPRESA'], $row['NRINSJURIEST'], $row['CDSERIENFE'], str_pad(($intAntigoCodigo+$ind), 9, 0, STR_PAD_LEFT), $row['TPAMBIENTE'], $row['CDINSCESTA'], $row['CDINSCMUNI'], $NRPROTOCINUTNFE, $IDTPEMISINUTIL, $DSJUSNUMINUTILNFE);
            }
            $descricao = 'Numero Anterior: ' . str_pad(($intAntigoCodigo+1), 9, 0, STR_PAD_LEFT) . ' Alterado para: ' . str_pad(($intNovoCodigo), 9, 0, STR_PAD_LEFT);
            $tipo = '2';
            $this->numeracaoTransmissaoNfeFactory->insereLogOpera($row['NMOPERADOR'], $tipo, $cdcontador, $descricao, $row['CDOPERADOR']) ;
            
            $connection->commit();
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
}