<?php
namespace Teknisa\DF\Services;

use Teknisa\DF\Factories\Ger98700ControleFiscalFactory;
use Teknisa\Libs\Util\Environment;

class Ger98700ControleFiscalService {

    protected $controleFiscalFactory;
    protected $environment;

    public function __construct(Environment $environment, Ger98700ControleFiscalFactory $controleFiscalFactory) {
        $this->controleFiscalFactory = $controleFiscalFactory;
        $this->environment = $environment;
    }

    public function validateTipooperac($row) {
        $nrorg = $this->environment->getNrorgTrab();
        if (empty($row["IDENTRSAIDOP"])) {
            throw new \Exception("Entrada/Saída deve ser informado.");
        }
        if (empty($row["CDTIPOOPER"])) {
            throw new \Exception("Tipo de operação deve ser informado.");
        }
        if ($row["IDENTRSAIDOP"] == 'E') {
            if ($row["IDOBINFCSTICMENT"] == 'S') {
                $idperaltcstfenf = $this->controleFiscalFactory->getIdperaltcstfenf(array("NRORG" => $nrorg))["IDPERALTCSTFENF"];
                if ($idperaltcstfenf != 'S') {
                    throw new \Exception("Operação bloqueada! Alteração manual do CST não permitida. Favor verificar o parâmetro \"Permite alterar manualmente o CST no fechamento de notas fiscais\".");
                }
            }
        }
        else {
            if ($row["IDGERICMSTDESPA"] == 'S' && empty($row["CDENCARGOICMST"])) {
                throw new \Exception("Encargo deve ser informado.");
            }
            if ($row["IDGERIPIDESPA"] == 'S' && empty($row["CDIMPOSTOIPI"])) {
                throw new \Exception("Imposto deve ser informado.");
            }
            if (!empty($row["CDENCARGOICMST"]) || !empty($row["CDIMPOSTOIPI"])) {
                $impostosObrigatorios = $this->controleFiscalFactory->getImpostosObrigatorios(array(
                    "IDENTRSAIDOP" => $row["IDENTRSAIDOP"],
                    "CDTIPOOPER" => $row["CDTIPOOPER"],
                    "NRORG" => $nrorg
                ));
                if (!empty($row["CDENCARGOICMST"])) {
                    foreach ($impostosObrigatorios as $imposto) {
                        $impostoSubstTribut = $this->controleFiscalFactory->getImpostoSubstTribut(array(
                            "CDIMPOSTO" => $imposto["CDIMPOSTO"],
                            "NRORG" => $nrorg
                        ));
                        if ($impostoSubstTribut != NULL) {
                            throw new \Exception("Operação bloqueada! Imposto de substituição tributária está relacionado como Imposto Obrigatório.");
                        }
                    }
                }
                if (!empty($row["CDIMPOSTOIPI"])) {
                    foreach ($impostosObrigatorios as $imposto) {
                        if ($row["CDIMPOSTOIPI"] == $imposto["CDIMPOSTO"]) {
                            throw new \Exception("Operação bloqueada! Imposto IPI está relacionado como Imposto Obrigatório.");
                        }
                    }
                }
            }
        }
    }

    public function validateImpobrigtpoper($row) {
        $nrorg = $this->environment->getNrorgTrab();
        if (empty($row["IDENTRSAIDOP"])) {
            throw new \Exception("Entrada/Saída deve ser informado.");
        }
        if (empty($row["CDTIPOOPER"])) {
            throw new \Exception("Tipo de operação deve ser informado.");
        }
        if (empty($row["CDIMPOSTO"])) {
            throw new \Exception("Imposto deve ser informado.");
        }
        $impostosObrigatorios = $this->controleFiscalFactory->getImpostosObrigatorios(array(
            "IDENTRSAIDOP" => $row["IDENTRSAIDOP"],
            "CDTIPOOPER" => $row["CDTIPOOPER"],
            "NRORG" => $nrorg
        ));
        forEach($impostosObrigatorios as $impostoCadastrado) {
            if ($impostoCadastrado["CDIMPOSTO"] == $row["CDIMPOSTO"]) {
                throw new \Exception("Imposto já cadastrado.");
            }
        }
        if ($row["IDENTRSAIDOP"] == 'S') {
            $tipoOper = $this->controleFiscalFactory->getTipooperParameters(array(
                "IDENTRSAIDOP" => $row["IDENTRSAIDOP"],
                "CDTIPOOPER" => $row["CDTIPOOPER"],
                "NRORG" => $nrorg
            ));
            if ($tipoOper["IDGERIPIDESPA"] == 'S' && $tipoOper["CDIMPOSTOIPI"] == $row["CDIMPOSTO"]) {
                throw new \Exception("Operação bloqueada! Imposto IPI gerado como despesa acessória.");
            }
            $impostoSubstTribut = $this->controleFiscalFactory->getImpostoSubstTribut(array(
                "CDIMPOSTO" => $row["CDIMPOSTO"],
                "NRORG" => $nrorg
            ));
            if ($impostoSubstTribut != NULL) {
                throw new \Exception("Operação bloqueada! Imposto de substituição tributária gerado como despesa acessória.");
            }
        }
    }
}
